package com.qmth.teachcloud.mark.service.impl;

import java.util.List;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import com.qmth.teachcloud.common.enums.mark.MarkPaperStatus;
import com.qmth.teachcloud.common.service.TeachcloudCommonService;
import com.qmth.teachcloud.common.util.FileStoreUtil;
import com.qmth.teachcloud.mark.entity.MarkDocument;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.enums.DocumentType;
import com.qmth.teachcloud.mark.mapper.MarkDocumentMapper;
import com.qmth.teachcloud.mark.service.MarkDocumentService;
import com.qmth.teachcloud.mark.service.MarkPaperService;

/**
 * <p>
 * 考试科目表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkDocumentServiceImpl extends ServiceImpl<MarkDocumentMapper, MarkDocument> implements MarkDocumentService {
    @Resource
    private FileStoreUtil fileStoreUtil;
    @Resource
    private TeachcloudCommonService teachcloudCommonService;

    @Resource
    private MarkPaperService markPaperService;
    @Override
    public List<MarkDocument> listByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkDocument> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkDocument::getExamId, examId)
                .eq(MarkDocument::getPaperNumber, paperNumber);
        return this.list(queryWrapper);
    }

    @Override
    public String importById(Long id, MultipartFile file, String md5) {
        MarkDocument document = this.getById(id);
        if(document==null){
            throw ExceptionResultEnum.ERROR.exception("文档不存在");
        }
        if(document.getType().equals(DocumentType.CUSTOM)
                ||document.getType().equals(DocumentType.SYLLABUS)
                ||document.getType().equals(DocumentType.PROCESS_SCORE)){
            throw ExceptionResultEnum.ERROR.exception("文档类型不正确");
        }
        String filePath = fileStoreUtil.uploadFile(file, md5, UploadFileEnum.FILE);
        UpdateWrapper<MarkDocument> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkDocument::getFilePath, filePath)
                .set(MarkDocument::getFileCount, 1)
                .eq(MarkDocument::getId, id);
        this.update(updateWrapper);
        return teachcloudCommonService.filePreview(filePath);
    }

    @Override
    public void deleteById(Long id) {
        MarkDocument document = this.getById(id);
        if(document==null){
            throw ExceptionResultEnum.ERROR.exception("文档不存在");
        }
        if(document.getType().equals(DocumentType.CUSTOM)){
            throw ExceptionResultEnum.ERROR.exception("文档类型不正确");
        }
        this.removeById(id);
    }

    @Override
    public String save(Long examId, String paperNumber, String documentName, MultipartFile file, String md5) {
        MarkPaper markPaper=markPaperService.getByExamIdAndPaperNumber(examId,paperNumber);
        if(markPaper==null ||markPaper.getStatus().equals(MarkPaperStatus.FINISH)){
            throw ExceptionResultEnum.ERROR.exception("评卷未结束或试卷不存在");
        }
        MarkDocument document = new MarkDocument();
        document.setExamId(examId);
        document.setPaperNumber(paperNumber);
        document.setCourseCode(markPaper.getCourseCode());
        document.setCourseName(markPaper.getCourseName());
        document.setType(DocumentType.CUSTOM);
        document.setFileCount(1);
        String filePath = fileStoreUtil.uploadFile(file, md5, UploadFileEnum.FILE);
        document.setFilePath(filePath);
        return teachcloudCommonService.filePreview(filePath);
    }
}
